
/*  A Bison parser, made from parser.y  */

#define YYDEBUG 1

#define YYBISON 1  /* Identify Bison output.  */

#define	tINTEGER	258
#define	tOBJECT	259
#define	tFLOAT	260
#define	tSTRING	261
#define	tID	262
#define	tERROR	263
#define	tIF	264
#define	tELSE	265
#define	tELSEIF	266
#define	tENDIF	267
#define	tFOR	268
#define	tIN	269
#define	tENDFOR	270
#define	tRETURN	271
#define	tFORK	272
#define	tENDFORK	273
#define	tWHILE	274
#define	tENDWHILE	275
#define	tTRY	276
#define	tENDTRY	277
#define	tEXCEPT	278
#define	tFINALLY	279
#define	tANY	280
#define	tBREAK	281
#define	tCONTINUE	282
#define	tTO	283
#define	tARROW	284
#define	tOR	285
#define	tAND	286
#define	tEQ	287
#define	tNE	288
#define	tLE	289
#define	tGE	290
#define	tUNARYMINUS	291


/******************************************************************************
  Copyright (c) 1992, 1995, 1996 Xerox Corporation.  All rights reserved.
  Portions of this code were written by Stephen White, aka ghond.
  Use and copying of this software and preparation of derivative works based
  upon this software are permitted.  Any distribution of this software or
  derivative works must comply with all applicable United States export
  control laws.  This software is made available AS IS, and Xerox Corporation
  makes no warranty about the software, its performance or its conformity to
  any specification.  Any person obtaining a copy of this software is requested
  to send their name and post office or electronic mail address to:
    Pavel Curtis
    Xerox PARC
    3333 Coyote Hill Rd.
    Palo Alto, CA 94304
    Pavel@Xerox.Com
 *****************************************************************************/

/* Modified by Christopher Unkel for WinMOO: */
/**********************************************************************
 Copyright (c) 1996, 1997 Christopher Unkel.  All rights reserved.
 
 Use and copying of the software and preparation and distribution of
 derivative works is permitted provided this copyright notice is
 left intact.  Any distribution of this software or derivative works
 must comply with all applicable United States export control laws.
 
 This software is provided AS IS, without express or implied warranty
 of any kind.  Christopher Unkel makes no representations concerning
 either the merchantability of this software or the suitability of
 this software of any particular purpose.
 **********************************************************************/

/*************************************************************************/
/* NOTE: If you add an #include here, make sure you properly update the  */
/*       parser.o dependency line in the Makefile.			 */
/*************************************************************************/

#include "my-ctype.h"
#include "my-math.h"
#include "my-stdlib.h"
#include "my-string.h"

#include "ast.h"
#include "code_gen.h" 
#include "config.h"
#include "functions.h"
#include "keywords.h"
#include "list.h"
#include "log.h"
#include "numbers.h"
#include "opcode.h"
#include "parser.h"
#include "program.h"
#include "storage.h"
#include "streams.h"
#include "structures.h"
#include "sym_table.h"
#include "utils.h"
#include "version.h" 

static Stmt    	       *prog_start;
static int		dollars_ok;
static DB_Version	language_version;

static void	error(char *s, char *t);
static void	warning(char *s, char *t);
static int	find_id(char *name);
static void yyerror(char *s);
static int	yylex(void);
static Scatter *scatter_from_arglist(Arg_List *);
static Scatter *add_scatter_item(Scatter *, Scatter *);
static void	vet_scatter(Scatter *);
static void	push_loop_name(char *);
static void	pop_loop_name(void);
static void	suspend_loop_scope(void);
static void	resume_loop_scope(void);

static void null_func(void *p, char *c);

enum loop_exit_kind { LOOP_BREAK, LOOP_CONTINUE };

static void	check_loop_name(char *s, enum loop_exit_kind);

typedef union {
  Stmt	       *stmt;
  Expr	       *expr;
  int		integer;
  Objid		object;
  double       *real;
  char	       *string;
  enum error	error;
  Arg_List     *args;
  Cond_Arm     *arm;
  Except_Arm   *except;
  Scatter      *scatter;
} YYSTYPE;

#ifndef YYLTYPE
typedef
  struct yyltype
    {
      int timestamp;
      int first_line;
      int first_column;
      int last_line;
      int last_column;
      char *text;
   }
  yyltype;

#define YYLTYPE yyltype
#endif

#include <stdio.h>

#ifndef __STDC__
#define const
#endif



#define	YYFINAL		216
#define	YYFLAG		-32768
#define	YYNTBASE	63

#define YYTRANSLATE(x) ((unsigned)(x) <= 291 ? yytranslate[x] : 86)

static const char yytranslate[] = {     0,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,    47,     2,     2,    52,    45,     2,    60,    53,
    54,    43,    41,    62,    42,    49,    44,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,    50,    56,    37,
    30,    39,    31,    61,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
    51,     2,    55,    46,     2,    59,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,    57,    32,    58,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     1,     2,     3,     4,     5,
     6,     7,     8,     9,    10,    11,    12,    13,    14,    15,
    16,    17,    18,    19,    20,    21,    22,    23,    24,    25,
    26,    27,    28,    29,    33,    34,    35,    36,    38,    40,
    48
};

#if YYDEBUG != 0
static const short yyprhs[] = {     0,
     0,     2,     3,     6,    15,    16,    26,    27,    39,    40,
    48,    49,    58,    59,    67,    68,    77,    80,    83,    87,
    90,    94,    98,   101,   103,   108,   114,   115,   122,   123,
   126,   129,   130,   135,   141,   142,   144,   146,   148,   150,
   152,   154,   156,   159,   163,   169,   176,   182,   191,   197,
   205,   207,   211,   217,   222,   226,   230,   234,   238,   242,
   246,   250,   254,   258,   262,   266,   270,   274,   278,   282,
   285,   288,   292,   296,   302,   309,   310,   312,   314,   315,
   318,   319,   321,   323,   326,   330,   335,   339,   343,   347,
   352,   354,   357
};

#endif

static const short yyrhs[] = {    64,
     0,     0,    64,    65,     0,     9,    53,    78,    54,    64,
    72,    73,    12,     0,     0,    13,     7,    14,    53,    78,
    54,    66,    64,    15,     0,     0,    13,     7,    14,    51,
    78,    28,    78,    55,    67,    64,    15,     0,     0,    19,
    53,    78,    54,    68,    64,    20,     0,     0,    19,     7,
    53,    78,    54,    69,    64,    20,     0,     0,    17,    53,
    78,    54,    70,    64,    18,     0,     0,    17,     7,    53,
    78,    54,    71,    64,    18,     0,    78,    56,     0,    26,
    56,     0,    26,     7,    56,     0,    27,    56,     0,    27,
     7,    56,     0,    16,    78,    56,     0,    16,    56,     0,
    56,     0,    21,    64,    74,    22,     0,    21,    64,    24,
    64,    22,     0,     0,    72,    11,    53,    78,    54,    64,
     0,     0,    10,    64,     0,    23,    76,     0,     0,    74,
    23,    75,    76,     0,    77,    53,    80,    54,    64,     0,
     0,     7,     0,     3,     0,     5,     0,     6,     0,     4,
     0,     8,     0,     7,     0,    52,     7,     0,    78,    49,
     7,     0,    78,    49,    53,    78,    54,     0,    78,    50,
     7,    53,    82,    54,     0,    52,     7,    53,    82,    54,
     0,    78,    50,    53,    78,    54,    53,    82,    54,     0,
    78,    51,    79,    78,    55,     0,    78,    51,    79,    78,
    28,    78,    55,     0,    52,     0,    78,    30,    78,     0,
    57,    84,    58,    30,    78,     0,     7,    53,    82,    54,
     0,    78,    41,    78,     0,    78,    42,    78,     0,    78,
    43,    78,     0,    78,    44,    78,     0,    78,    45,    78,
     0,    78,    46,    78,     0,    78,    34,    78,     0,    78,
    33,    78,     0,    78,    35,    78,     0,    78,    36,    78,
     0,    78,    37,    78,     0,    78,    38,    78,     0,    78,
    39,    78,     0,    78,    40,    78,     0,    78,    14,    78,
     0,    42,    78,     0,    47,    78,     0,    53,    78,    54,
     0,    57,    82,    58,     0,    78,    31,    78,    32,    78,
     0,    59,    78,    47,    80,    81,    60,     0,     0,    25,
     0,    83,     0,     0,    29,    78,     0,     0,    83,     0,
    78,     0,    61,    78,     0,    83,    62,    78,     0,    83,
    62,    61,    78,     0,    83,    62,    85,     0,    84,    62,
    85,     0,    84,    62,     7,     0,    84,    62,    61,     7,
     0,    85,     0,    31,     7,     0,    31,     7,    30,    78,
     0
};

#if YYDEBUG != 0
static const short yyrline[] = { 0,
   113,   117,   120,   134,   143,   147,   155,   159,   168,   172,
   180,   184,   192,   196,   204,   208,   216,   221,   227,   233,
   239,   245,   250,   255,   257,   263,   271,   274,   290,   293,
   297,   300,   315,   325,   329,   332,   335,   341,   346,   351,
   356,   361,   366,   375,   382,   386,   393,   402,   406,   411,
   419,   425,   447,   455,   476,   480,   484,   488,   492,   496,
   500,   504,   508,   512,   516,   520,   524,   528,   532,   536,
   557,   562,   564,   569,   576,   585,   589,   592,   595,   598,
   601,   604,   608,   611,   613,   627,   643,   653,   657,   662,
   667,   671,   676

};

static const char * const yytname[] = {   "$","error","$illegal.","tINTEGER",
"tOBJECT","tFLOAT","tSTRING","tID","tERROR","tIF","tELSE","tELSEIF","tENDIF",
"tFOR","tIN","tENDFOR","tRETURN","tFORK","tENDFORK","tWHILE","tENDWHILE","tTRY",
"tENDTRY","tEXCEPT","tFINALLY","tANY","tBREAK","tCONTINUE","tTO","tARROW","'='",
"'?'","'|'","tOR","tAND","tEQ","tNE","'<'","tLE","'>'","tGE","'+'","'-'","'*'",
"'/'","'%'","'^'","'!'","tUNARYMINUS","'.'","':'","'['","'$'","'('","')'","']'",
"';'","'{'","'}'","'`'","'\\''","'@'","','","program","statements","statement",
"@1","@2","@3","@4","@5","@6","elseifs","elsepart","excepts","@7","except","opt_id",
"expr","dollars_up","codes","default","arglist","ne_arglist","scatter","scatter_item",
""
};
#endif

static const short yyr1[] = {     0,
    63,    64,    64,    65,    66,    65,    67,    65,    68,    65,
    69,    65,    70,    65,    71,    65,    65,    65,    65,    65,
    65,    65,    65,    65,    65,    65,    72,    72,    73,    73,
    74,    75,    74,    76,    77,    77,    78,    78,    78,    78,
    78,    78,    78,    78,    78,    78,    78,    78,    78,    78,
    78,    78,    78,    78,    78,    78,    78,    78,    78,    78,
    78,    78,    78,    78,    78,    78,    78,    78,    78,    78,
    78,    78,    78,    78,    78,    79,    80,    80,    81,    81,
    82,    82,    83,    83,    83,    83,    84,    84,    84,    84,
    84,    85,    85
};

static const short yyr2[] = {     0,
     1,     0,     2,     8,     0,     9,     0,    11,     0,     7,
     0,     8,     0,     7,     0,     8,     2,     2,     3,     2,
     3,     3,     2,     1,     4,     5,     0,     6,     0,     2,
     2,     0,     4,     5,     0,     1,     1,     1,     1,     1,
     1,     1,     2,     3,     5,     6,     5,     8,     5,     7,
     1,     3,     5,     4,     3,     3,     3,     3,     3,     3,
     3,     3,     3,     3,     3,     3,     3,     3,     3,     2,
     2,     3,     3,     5,     6,     0,     1,     1,     0,     2,
     0,     1,     1,     2,     3,     4,     3,     3,     3,     4,
     1,     2,     4
};

static const short yydefact[] = {     2,
     1,    37,    40,    38,    39,    42,    41,     0,     0,     0,
     0,     0,     2,     0,     0,     0,     0,    51,     0,    24,
    81,     0,     3,     0,    81,     0,     0,    23,     0,     0,
     0,     0,     0,     0,     0,    18,     0,    20,    70,    71,
    43,     0,     0,     0,    83,     0,    82,     0,    91,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,    76,
    17,     0,    82,     0,     0,    22,     0,     0,     0,     0,
    35,     2,     0,    19,    21,    81,    72,    92,    84,    73,
     0,     0,     0,     0,    69,    52,     0,    62,    61,    63,
    64,    65,    66,    67,    68,    55,    56,    57,    58,    59,
    60,    44,     0,     0,     0,     0,    54,     0,     2,     0,
     0,     0,    13,     0,     9,    36,    31,     0,     0,    25,
    32,     0,     0,     0,    85,    87,     0,    89,     0,    88,
    77,    79,    78,     0,     0,    81,     0,     0,    27,     0,
     0,    15,     2,    11,     2,     0,    26,    35,    47,    93,
    86,    53,    90,     0,     0,    74,    45,     0,     0,     0,
    49,    29,     0,     5,     2,     0,     2,     0,     0,    33,
    80,    75,    46,    81,     0,     2,     0,     0,     0,     2,
     0,    14,     0,    10,     2,     0,    50,    30,     0,     4,
     7,     0,    16,    12,    34,    48,     0,     2,     6,     2,
     0,    28,     8,     0,     0,     0
};

static const short yydefgoto[] = {   214,
     1,    23,   190,   208,   155,   177,   153,   175,   172,   188,
    83,   158,   127,   128,    24,   116,   142,   165,    46,    73,
    48,    49
};

static const short yypact[] = {-32768,
   754,-32768,-32768,-32768,-32768,   -35,-32768,   -26,    -6,   331,
    12,    17,-32768,     8,    15,   445,   445,    28,   445,-32768,
   149,   445,-32768,   784,    26,   445,    30,-32768,   817,    13,
   445,    22,   445,   298,    21,-32768,    24,-32768,    -9,    -9,
    35,   949,    96,   445,  1379,    46,    43,   -52,-32768,  1279,
   445,   445,   445,   445,   445,   445,   445,   445,   445,   445,
   445,   445,   445,   445,   445,   445,   445,    29,    31,-32768,
-32768,    53,    47,   982,   -28,-32768,   445,  1015,   445,  1048,
   105,-32768,    74,-32768,-32768,    26,-32768,    84,  1379,-32768,
   180,    87,     6,   209,    49,  1379,  1313,  1430,  1430,    49,
    49,    49,    49,    49,    49,    75,    75,   -42,   -42,   -42,
   -42,-32768,   445,    69,   445,   445,-32768,   239,-32768,   445,
   445,  1081,-32768,  1114,-32768,-32768,-32768,    77,   355,-32768,
-32768,    73,   445,   445,  1379,-32768,   445,-32768,   124,-32768,
-32768,   104,    47,   445,  1147,    26,  1180,   850,   754,  1346,
  1213,-32768,-32768,-32768,-32768,   209,-32768,   105,-32768,  1379,
  1379,  1379,-32768,   445,    76,  1412,-32768,    80,    82,   445,
-32768,    91,   445,-32768,-32768,   412,-32768,   469,    83,-32768,
  1379,-32768,-32768,    26,   883,-32768,    90,   132,   916,-32768,
   526,-32768,   583,-32768,-32768,    92,-32768,   754,   445,-32768,
-32768,   640,-32768,-32768,   754,-32768,  1246,-32768,-32768,-32768,
   697,   754,-32768,   145,   147,-32768
};

static const short yypgoto[] = {-32768,
   -13,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,-32768,
-32768,-32768,   -10,-32768,    -5,-32768,    -7,-32768,   -23,   -18,
-32768,   -48
};


#define	YYLAST		1481


static const short yytable[] = {    34,
    27,    72,    47,    67,    29,    92,    68,    69,    70,    93,
    39,    40,   138,    42,    35,    45,    50,    25,    30,    45,
    74,    37,   120,    32,   121,    78,    26,    80,     2,     3,
     4,     5,     6,     7,    41,   112,    43,   114,    89,    68,
    69,    70,   136,    75,   140,    95,    96,    97,    98,    99,
   100,   101,   102,   103,   104,   105,   106,   107,   108,   109,
   110,   111,   132,    36,    31,    77,   139,    16,   129,    33,
    38,   122,    17,   124,    79,   143,    84,    18,    19,    85,
    45,   113,    21,   115,    22,   135,    44,    86,    45,    62,
    63,    64,    65,    66,    67,   130,   131,    68,    69,    70,
   186,   187,    88,    90,    91,   149,   117,   145,   118,   147,
   148,   126,   135,   133,   150,   151,   137,    64,    65,    66,
    67,   146,   168,    68,    69,    70,   159,   160,   161,   156,
   163,   162,   164,   183,   184,   182,   195,   143,   166,   176,
    45,   178,   199,   200,   215,   206,   216,   180,   179,     0,
    45,     2,     3,     4,     5,     6,     7,     0,   181,     0,
   196,   191,     0,   193,   185,     0,     0,   189,     0,     0,
     0,     0,   198,     0,     0,     0,   202,     0,    45,    43,
     0,   205,     2,     3,     4,     5,     6,     7,     0,     0,
    16,     0,     0,   207,   211,    17,   212,     0,     0,     0,
    18,    19,     0,     0,     0,    21,     0,    22,     0,    44,
    43,     2,     3,     4,     5,     6,     7,     0,     0,     0,
     0,    16,     0,     0,     0,     0,    17,     0,     0,     0,
     0,    18,    19,   141,     0,     0,    21,     0,    22,     0,
   134,     2,     3,     4,     5,     6,     7,     0,     0,     0,
    16,     0,     0,     0,     0,    17,     0,     0,     0,     0,
    18,    19,     0,     0,     0,    21,     0,    22,     0,    44,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
    16,     0,     0,     0,     0,    17,     0,     0,     0,     0,
    18,    19,     0,     0,     0,    21,     0,    22,     0,   134,
     2,     3,     4,     5,     6,     7,     8,     0,     0,     0,
     9,     0,     0,    10,    11,     0,    12,     0,    13,     0,
    81,    82,     0,    14,    15,     0,     0,     0,     0,     0,
     0,     0,     0,     2,     3,     4,     5,     6,     7,    16,
     0,     0,     0,     0,    17,     0,     0,     0,     0,    18,
    19,     0,     0,    20,    21,     0,    22,     2,     3,     4,
     5,     6,     7,     8,     0,     0,     0,     9,     0,     0,
    10,    11,    16,    12,     0,    13,   157,    17,     0,     0,
    14,    15,    18,    19,     0,     0,    28,    21,     0,    22,
     0,     0,     0,     0,     0,     0,    16,     0,     0,     0,
     0,    17,     0,     0,     0,     0,    18,    19,     0,     0,
    20,    21,     0,    22,     2,     3,     4,     5,     6,     7,
     8,     0,     0,     0,     9,     0,     0,    10,    11,   192,
    12,     0,    13,     0,     0,     0,     0,    14,    15,     0,
     0,     0,     0,     0,     0,     0,     0,     2,     3,     4,
     5,     6,     7,    16,     0,     0,     0,     0,    17,     0,
     0,     0,     0,    18,    19,     0,     0,    20,    21,     0,
    22,     2,     3,     4,     5,     6,     7,     8,     0,     0,
     0,     9,     0,     0,    10,    11,    16,    12,   194,    13,
     0,    17,     0,     0,    14,    15,    18,    19,     0,     0,
     0,    21,     0,    22,     0,     0,     0,     0,     0,     0,
    16,     0,     0,     0,     0,    17,     0,     0,     0,     0,
    18,    19,     0,     0,    20,    21,     0,    22,     2,     3,
     4,     5,     6,     7,     8,     0,     0,     0,     9,     0,
     0,    10,    11,   203,    12,     0,    13,     0,     0,     0,
     0,    14,    15,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,    16,     0,     0,
     0,     0,    17,     0,     0,     0,     0,    18,    19,     0,
     0,    20,    21,     0,    22,     2,     3,     4,     5,     6,
     7,     8,     0,     0,     0,     9,     0,     0,    10,    11,
     0,    12,   204,    13,     0,     0,     0,     0,    14,    15,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,    16,     0,     0,     0,     0,    17,
     0,     0,     0,     0,    18,    19,     0,     0,    20,    21,
     0,    22,     2,     3,     4,     5,     6,     7,     8,     0,
     0,     0,     9,     0,   209,    10,    11,     0,    12,     0,
    13,     0,     0,     0,     0,    14,    15,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,    16,     0,     0,     0,     0,    17,     0,     0,     0,
     0,    18,    19,     0,     0,    20,    21,     0,    22,     2,
     3,     4,     5,     6,     7,     8,     0,     0,     0,     9,
     0,   213,    10,    11,     0,    12,     0,    13,     0,     0,
     0,     0,    14,    15,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,    16,     0,
     0,     0,     0,    17,     0,     0,     0,     0,    18,    19,
     0,     0,    20,    21,     0,    22,     2,     3,     4,     5,
     6,     7,     8,     0,     0,     0,     9,     0,     0,    10,
    11,     0,    12,     0,    13,     0,     0,     0,     0,    14,
    15,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,    16,     0,    51,     0,     0,
    17,     0,     0,     0,     0,    18,    19,     0,     0,    20,
    21,     0,    22,    52,    53,     0,    54,    55,    56,    57,
    58,    59,    60,    61,    62,    63,    64,    65,    66,    67,
    51,     0,    68,    69,    70,     0,     0,     0,     0,    71,
     0,     0,     0,     0,     0,     0,    52,    53,     0,    54,
    55,    56,    57,    58,    59,    60,    61,    62,    63,    64,
    65,    66,    67,    51,     0,    68,    69,    70,     0,     0,
     0,     0,    76,     0,     0,     0,     0,   170,     0,    52,
    53,     0,    54,    55,    56,    57,    58,    59,    60,    61,
    62,    63,    64,    65,    66,    67,    51,     0,    68,    69,
    70,     0,     0,     0,   171,     0,     0,     0,     0,     0,
     0,     0,    52,    53,     0,    54,    55,    56,    57,    58,
    59,    60,    61,    62,    63,    64,    65,    66,    67,    51,
     0,    68,    69,    70,     0,     0,     0,   197,     0,     0,
     0,     0,     0,     0,     0,    52,    53,     0,    54,    55,
    56,    57,    58,    59,    60,    61,    62,    63,    64,    65,
    66,    67,    51,     0,    68,    69,    70,     0,     0,     0,
   201,     0,     0,     0,     0,     0,     0,     0,    52,    53,
     0,    54,    55,    56,    57,    58,    59,    60,    61,    62,
    63,    64,    65,    66,    67,    51,     0,    68,    69,    70,
     0,     0,    87,     0,     0,     0,     0,     0,     0,     0,
     0,    52,    53,     0,    54,    55,    56,    57,    58,    59,
    60,    61,    62,    63,    64,    65,    66,    67,    51,     0,
    68,    69,    70,     0,     0,   119,     0,     0,     0,     0,
     0,     0,     0,     0,    52,    53,     0,    54,    55,    56,
    57,    58,    59,    60,    61,    62,    63,    64,    65,    66,
    67,    51,     0,    68,    69,    70,     0,     0,   123,     0,
     0,     0,     0,     0,     0,     0,     0,    52,    53,     0,
    54,    55,    56,    57,    58,    59,    60,    61,    62,    63,
    64,    65,    66,    67,    51,     0,    68,    69,    70,     0,
     0,   125,     0,     0,     0,     0,     0,     0,     0,     0,
    52,    53,     0,    54,    55,    56,    57,    58,    59,    60,
    61,    62,    63,    64,    65,    66,    67,    51,     0,    68,
    69,    70,     0,     0,   152,     0,     0,     0,     0,     0,
     0,     0,     0,    52,    53,     0,    54,    55,    56,    57,
    58,    59,    60,    61,    62,    63,    64,    65,    66,    67,
    51,     0,    68,    69,    70,     0,     0,   154,     0,     0,
     0,     0,     0,     0,     0,     0,    52,    53,     0,    54,
    55,    56,    57,    58,    59,    60,    61,    62,    63,    64,
    65,    66,    67,    51,     0,    68,    69,    70,     0,     0,
   167,     0,     0,     0,     0,     0,     0,     0,     0,    52,
    53,     0,    54,    55,    56,    57,    58,    59,    60,    61,
    62,    63,    64,    65,    66,    67,    51,     0,    68,    69,
    70,     0,     0,   169,     0,     0,     0,     0,     0,     0,
     0,     0,    52,    53,     0,    54,    55,    56,    57,    58,
    59,    60,    61,    62,    63,    64,    65,    66,    67,    51,
     0,    68,    69,    70,     0,     0,   174,     0,     0,     0,
     0,     0,     0,     0,     0,    52,    53,     0,    54,    55,
    56,    57,    58,    59,    60,    61,    62,    63,    64,    65,
    66,    67,    51,     0,    68,    69,    70,     0,     0,   210,
     0,     0,     0,     0,     0,     0,     0,     0,    52,    53,
     0,    54,    55,    56,    57,    58,    59,    60,    61,    62,
    63,    64,    65,    66,    67,    94,    51,    68,    69,    70,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,    52,    53,   144,    54,    55,    56,    57,    58,
    59,    60,    61,    62,    63,    64,    65,    66,    67,    51,
     0,    68,    69,    70,     0,     0,     0,     0,     0,     0,
     0,     0,     0,   173,     0,    52,    53,     0,    54,    55,
    56,    57,    58,    59,    60,    61,    62,    63,    64,    65,
    66,    67,    51,     0,    68,    69,    70,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,    52,    53,
     0,    54,    55,    56,    57,    58,    59,    60,    61,    62,
    63,    64,    65,    66,    67,    51,     0,    68,    69,    70,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,-32768,    51,    54,    55,    56,    57,    58,    59,
    60,    61,    62,    63,    64,    65,    66,    67,     0,     0,
    68,    69,    70,     0,    56,    57,    58,    59,    60,    61,
    62,    63,    64,    65,    66,    67,     0,     0,    68,    69,
    70
};

static const short yycheck[] = {    13,
     7,    25,    21,    46,    10,    58,    49,    50,    51,    62,
    16,    17,     7,    19,     7,    21,    22,    53,     7,    25,
    26,     7,    51,     7,    53,    31,    53,    33,     3,     4,
     5,     6,     7,     8,     7,     7,    31,     7,    44,    49,
    50,    51,    91,    14,    93,    51,    52,    53,    54,    55,
    56,    57,    58,    59,    60,    61,    62,    63,    64,    65,
    66,    67,    86,    56,    53,    53,    61,    42,    82,    53,
    56,    77,    47,    79,    53,    94,    56,    52,    53,    56,
    86,    53,    57,    53,    59,    91,    61,    53,    94,    41,
    42,    43,    44,    45,    46,    22,    23,    49,    50,    51,
    10,    11,     7,    58,    62,   119,    54,   113,    62,   115,
   116,     7,   118,    30,   120,   121,    30,    43,    44,    45,
    46,    53,   146,    49,    50,    51,    54,   133,   134,    53,
     7,   137,    29,    54,    53,    60,    54,   156,   144,   153,
   146,   155,    53,    12,     0,    54,     0,   158,   156,    -1,
   156,     3,     4,     5,     6,     7,     8,    -1,   164,    -1,
   184,   175,    -1,   177,   170,    -1,    -1,   173,    -1,    -1,
    -1,    -1,   186,    -1,    -1,    -1,   190,    -1,   184,    31,
    -1,   195,     3,     4,     5,     6,     7,     8,    -1,    -1,
    42,    -1,    -1,   199,   208,    47,   210,    -1,    -1,    -1,
    52,    53,    -1,    -1,    -1,    57,    -1,    59,    -1,    61,
    31,     3,     4,     5,     6,     7,     8,    -1,    -1,    -1,
    -1,    42,    -1,    -1,    -1,    -1,    47,    -1,    -1,    -1,
    -1,    52,    53,    25,    -1,    -1,    57,    -1,    59,    -1,
    61,     3,     4,     5,     6,     7,     8,    -1,    -1,    -1,
    42,    -1,    -1,    -1,    -1,    47,    -1,    -1,    -1,    -1,
    52,    53,    -1,    -1,    -1,    57,    -1,    59,    -1,    61,
    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
    42,    -1,    -1,    -1,    -1,    47,    -1,    -1,    -1,    -1,
    52,    53,    -1,    -1,    -1,    57,    -1,    59,    -1,    61,
     3,     4,     5,     6,     7,     8,     9,    -1,    -1,    -1,
    13,    -1,    -1,    16,    17,    -1,    19,    -1,    21,    -1,
    23,    24,    -1,    26,    27,    -1,    -1,    -1,    -1,    -1,
    -1,    -1,    -1,     3,     4,     5,     6,     7,     8,    42,
    -1,    -1,    -1,    -1,    47,    -1,    -1,    -1,    -1,    52,
    53,    -1,    -1,    56,    57,    -1,    59,     3,     4,     5,
     6,     7,     8,     9,    -1,    -1,    -1,    13,    -1,    -1,
    16,    17,    42,    19,    -1,    21,    22,    47,    -1,    -1,
    26,    27,    52,    53,    -1,    -1,    56,    57,    -1,    59,
    -1,    -1,    -1,    -1,    -1,    -1,    42,    -1,    -1,    -1,
    -1,    47,    -1,    -1,    -1,    -1,    52,    53,    -1,    -1,
    56,    57,    -1,    59,     3,     4,     5,     6,     7,     8,
     9,    -1,    -1,    -1,    13,    -1,    -1,    16,    17,    18,
    19,    -1,    21,    -1,    -1,    -1,    -1,    26,    27,    -1,
    -1,    -1,    -1,    -1,    -1,    -1,    -1,     3,     4,     5,
     6,     7,     8,    42,    -1,    -1,    -1,    -1,    47,    -1,
    -1,    -1,    -1,    52,    53,    -1,    -1,    56,    57,    -1,
    59,     3,     4,     5,     6,     7,     8,     9,    -1,    -1,
    -1,    13,    -1,    -1,    16,    17,    42,    19,    20,    21,
    -1,    47,    -1,    -1,    26,    27,    52,    53,    -1,    -1,
    -1,    57,    -1,    59,    -1,    -1,    -1,    -1,    -1,    -1,
    42,    -1,    -1,    -1,    -1,    47,    -1,    -1,    -1,    -1,
    52,    53,    -1,    -1,    56,    57,    -1,    59,     3,     4,
     5,     6,     7,     8,     9,    -1,    -1,    -1,    13,    -1,
    -1,    16,    17,    18,    19,    -1,    21,    -1,    -1,    -1,
    -1,    26,    27,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
    -1,    -1,    -1,    -1,    -1,    -1,    -1,    42,    -1,    -1,
    -1,    -1,    47,    -1,    -1,    -1,    -1,    52,    53,    -1,
    -1,    56,    57,    -1,    59,     3,     4,     5,     6,     7,
     8,     9,    -1,    -1,    -1,    13,    -1,    -1,    16,    17,
    -1,    19,    20,    21,    -1,    -1,    -1,    -1,    26,    27,
    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
    -1,    -1,    -1,    -1,    42,    -1,    -1,    -1,    -1,    47,
    -1,    -1,    -1,    -1,    52,    53,    -1,    -1,    56,    57,
    -1,    59,     3,     4,     5,     6,     7,     8,     9,    -1,
    -1,    -1,    13,    -1,    15,    16,    17,    -1,    19,    -1,
    21,    -1,    -1,    -1,    -1,    26,    27,    -1,    -1,    -1,
    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
    -1,    42,    -1,    -1,    -1,    -1,    47,    -1,    -1,    -1,
    -1,    52,    53,    -1,    -1,    56,    57,    -1,    59,     3,
     4,     5,     6,     7,     8,     9,    -1,    -1,    -1,    13,
    -1,    15,    16,    17,    -1,    19,    -1,    21,    -1,    -1,
    -1,    -1,    26,    27,    -1,    -1,    -1,    -1,    -1,    -1,
    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    42,    -1,
    -1,    -1,    -1,    47,    -1,    -1,    -1,    -1,    52,    53,
    -1,    -1,    56,    57,    -1,    59,     3,     4,     5,     6,
     7,     8,     9,    -1,    -1,    -1,    13,    -1,    -1,    16,
    17,    -1,    19,    -1,    21,    -1,    -1,    -1,    -1,    26,
    27,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
    -1,    -1,    -1,    -1,    -1,    42,    -1,    14,    -1,    -1,
    47,    -1,    -1,    -1,    -1,    52,    53,    -1,    -1,    56,
    57,    -1,    59,    30,    31,    -1,    33,    34,    35,    36,
    37,    38,    39,    40,    41,    42,    43,    44,    45,    46,
    14,    -1,    49,    50,    51,    -1,    -1,    -1,    -1,    56,
    -1,    -1,    -1,    -1,    -1,    -1,    30,    31,    -1,    33,
    34,    35,    36,    37,    38,    39,    40,    41,    42,    43,
    44,    45,    46,    14,    -1,    49,    50,    51,    -1,    -1,
    -1,    -1,    56,    -1,    -1,    -1,    -1,    28,    -1,    30,
    31,    -1,    33,    34,    35,    36,    37,    38,    39,    40,
    41,    42,    43,    44,    45,    46,    14,    -1,    49,    50,
    51,    -1,    -1,    -1,    55,    -1,    -1,    -1,    -1,    -1,
    -1,    -1,    30,    31,    -1,    33,    34,    35,    36,    37,
    38,    39,    40,    41,    42,    43,    44,    45,    46,    14,
    -1,    49,    50,    51,    -1,    -1,    -1,    55,    -1,    -1,
    -1,    -1,    -1,    -1,    -1,    30,    31,    -1,    33,    34,
    35,    36,    37,    38,    39,    40,    41,    42,    43,    44,
    45,    46,    14,    -1,    49,    50,    51,    -1,    -1,    -1,
    55,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    30,    31,
    -1,    33,    34,    35,    36,    37,    38,    39,    40,    41,
    42,    43,    44,    45,    46,    14,    -1,    49,    50,    51,
    -1,    -1,    54,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
    -1,    30,    31,    -1,    33,    34,    35,    36,    37,    38,
    39,    40,    41,    42,    43,    44,    45,    46,    14,    -1,
    49,    50,    51,    -1,    -1,    54,    -1,    -1,    -1,    -1,
    -1,    -1,    -1,    -1,    30,    31,    -1,    33,    34,    35,
    36,    37,    38,    39,    40,    41,    42,    43,    44,    45,
    46,    14,    -1,    49,    50,    51,    -1,    -1,    54,    -1,
    -1,    -1,    -1,    -1,    -1,    -1,    -1,    30,    31,    -1,
    33,    34,    35,    36,    37,    38,    39,    40,    41,    42,
    43,    44,    45,    46,    14,    -1,    49,    50,    51,    -1,
    -1,    54,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
    30,    31,    -1,    33,    34,    35,    36,    37,    38,    39,
    40,    41,    42,    43,    44,    45,    46,    14,    -1,    49,
    50,    51,    -1,    -1,    54,    -1,    -1,    -1,    -1,    -1,
    -1,    -1,    -1,    30,    31,    -1,    33,    34,    35,    36,
    37,    38,    39,    40,    41,    42,    43,    44,    45,    46,
    14,    -1,    49,    50,    51,    -1,    -1,    54,    -1,    -1,
    -1,    -1,    -1,    -1,    -1,    -1,    30,    31,    -1,    33,
    34,    35,    36,    37,    38,    39,    40,    41,    42,    43,
    44,    45,    46,    14,    -1,    49,    50,    51,    -1,    -1,
    54,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    30,
    31,    -1,    33,    34,    35,    36,    37,    38,    39,    40,
    41,    42,    43,    44,    45,    46,    14,    -1,    49,    50,
    51,    -1,    -1,    54,    -1,    -1,    -1,    -1,    -1,    -1,
    -1,    -1,    30,    31,    -1,    33,    34,    35,    36,    37,
    38,    39,    40,    41,    42,    43,    44,    45,    46,    14,
    -1,    49,    50,    51,    -1,    -1,    54,    -1,    -1,    -1,
    -1,    -1,    -1,    -1,    -1,    30,    31,    -1,    33,    34,
    35,    36,    37,    38,    39,    40,    41,    42,    43,    44,
    45,    46,    14,    -1,    49,    50,    51,    -1,    -1,    54,
    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    30,    31,
    -1,    33,    34,    35,    36,    37,    38,    39,    40,    41,
    42,    43,    44,    45,    46,    47,    14,    49,    50,    51,
    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
    -1,    -1,    30,    31,    32,    33,    34,    35,    36,    37,
    38,    39,    40,    41,    42,    43,    44,    45,    46,    14,
    -1,    49,    50,    51,    -1,    -1,    -1,    -1,    -1,    -1,
    -1,    -1,    -1,    28,    -1,    30,    31,    -1,    33,    34,
    35,    36,    37,    38,    39,    40,    41,    42,    43,    44,
    45,    46,    14,    -1,    49,    50,    51,    -1,    -1,    -1,
    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    30,    31,
    -1,    33,    34,    35,    36,    37,    38,    39,    40,    41,
    42,    43,    44,    45,    46,    14,    -1,    49,    50,    51,
    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
    -1,    -1,    31,    14,    33,    34,    35,    36,    37,    38,
    39,    40,    41,    42,    43,    44,    45,    46,    -1,    -1,
    49,    50,    51,    -1,    35,    36,    37,    38,    39,    40,
    41,    42,    43,    44,    45,    46,    -1,    -1,    49,    50,
    51
};
/* -*-C-*-  Note some compilers choke on comments on `#line' lines.  */

/* Skeleton output parser for bison,
   Copyright (C) 1984, 1989, 1990 Bob Corbett and Richard Stallman

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 1, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */


#ifndef alloca
#ifdef __GNUC__
#define alloca __builtin_alloca
#else /* not GNU C.  */
#if (!defined (__STDC__) && defined (sparc)) || defined (__sparc__)
#include <alloca.h>
#else /* not sparc */
#if defined (MSDOS) && !defined (__TURBOC__)
#include <malloc.h>
#else /* not MSDOS, or __TURBOC__ */
#if defined(_AIX)
#include <malloc.h>
 #pragma alloca
#endif /* not _AIX */
#endif /* not MSDOS, or __TURBOC__ */
#endif /* not sparc.  */
#endif /* not GNU C.  */
#endif /* alloca not defined.  */

#define alloca _alloca

/* This is the parser code that is written into each bison parser
  when the %semantic_parser declaration is not specified in the grammar.
  It was written by Richard Stallman by simplifying the hairy parser
  used when %semantic_parser is specified.  */

/* Note: there must be only one dollar sign in this file.
   It is replaced by the list of actions, each action
   as one case of the switch.  */

#define yyerrok		(yyerrstatus = 0)
#define yyclearin	(yychar = YYEMPTY)
#define YYEMPTY		-2
#define YYEOF		0
#define YYACCEPT	return(0)
#define YYABORT 	return(1)
#define YYERROR		goto yyerrlab1
/* Like YYERROR except do call yyerror.
   This remains here temporarily to ease the
   transition to the new meaning of YYERROR, for GCC.
   Once GCC version 2 has supplanted version 1, this can go.  */
#define YYFAIL		goto yyerrlab
#define YYRECOVERING()  (!!yyerrstatus)
#define YYBACKUP(token, value) \
do								\
  if (yychar == YYEMPTY && yylen == 1)				\
    { yychar = (token), yylval = (value);			\
      yychar1 = YYTRANSLATE (yychar);				\
      YYPOPSTACK;						\
      goto yybackup;						\
    }								\
  else								\
    { yyerror ("syntax error: cannot back up"); YYERROR; }	\
while (0)

#define YYTERROR	1
#define YYERRCODE	256

#ifndef YYPURE
#define YYLEX		yylex()
#endif

#ifdef YYPURE
#ifdef YYLSP_NEEDED
#define YYLEX		yylex(&yylval, &yylloc)
#else
#define YYLEX		yylex(&yylval)
#endif
#endif

/* If nonreentrant, generate the variables here */

#ifndef YYPURE

int	yychar;			/*  the lookahead symbol		*/
YYSTYPE	yylval;			/*  the semantic value of the		*/
				/*  lookahead symbol			*/

#ifdef YYLSP_NEEDED
YYLTYPE yylloc;			/*  location data for the lookahead	*/
				/*  symbol				*/
#endif

int yynerrs;			/*  number of parse errors so far       */
#endif  /* not YYPURE */

#if YYDEBUG != 0
int yydebug;			/*  nonzero means print parse trace	*/
/* Since this is uninitialized, it does not stop multiple parsers
   from coexisting.  */
#endif

/*  YYINITDEPTH indicates the initial size of the parser's stacks	*/

#ifndef	YYINITDEPTH
#define YYINITDEPTH 200
#endif

/*  YYMAXDEPTH is the maximum size the stacks can grow to
    (effective only if the built-in stack extension method is used).  */

#if YYMAXDEPTH == 0
#undef YYMAXDEPTH
#endif

#ifndef YYMAXDEPTH
#define YYMAXDEPTH 10000
#endif

#if __GNUC__ > 1		/* GNU C and GNU C++ define this.  */
#define __yy_bcopy(FROM,TO,COUNT)	__builtin_memcpy(TO,FROM,COUNT)
#else				/* not GNU C or C++ */
#ifndef __cplusplus

/* This is the most reliable way to avoid incompatibilities
   in available built-in functions on various systems.  */
static void
__yy_bcopy (from, to, count)
     char *from;
     char *to;
     int count;
{
  register char *f = from;
  register char *t = to;
  register int i = count;

  while (i-- > 0)
    *t++ = *f++;
}

#else /* __cplusplus */

/* This is the most reliable way to avoid incompatibilities
   in available built-in functions on various systems.  */
static void
__yy_bcopy (char *from, char *to, int count)
{
  register char *f = from;
  register char *t = to;
  register int i = count;

  while (i-- > 0)
    *t++ = *f++;
}

#endif
#endif

int
yyparse()
{
  register int yystate;
  register int yyn;
  register short *yyssp;
  register YYSTYPE *yyvsp;
  int yyerrstatus;	/*  number of tokens to shift before error messages enabled */
  int yychar1;		/*  lookahead token as an internal (translated) token number */

  short	yyssa[YYINITDEPTH];	/*  the state stack			*/
  YYSTYPE yyvsa[YYINITDEPTH];	/*  the semantic value stack		*/

  short *yyss = yyssa;		/*  refer to the stacks thru separate pointers */
  YYSTYPE *yyvs = yyvsa;	/*  to allow yyoverflow to reallocate them elsewhere */

#ifdef YYLSP_NEEDED
  YYLTYPE yylsa[YYINITDEPTH];	/*  the location stack			*/
  YYLTYPE *yyls = yylsa;
  YYLTYPE *yylsp;

#define YYPOPSTACK   (yyvsp--, yyssp--, yylsp--)
#else
#define YYPOPSTACK   (yyvsp--, yyssp--)
#endif

  int yystacksize = YYINITDEPTH;

#ifdef YYPURE
  int yychar;
  YYSTYPE yylval;
  int yynerrs;
#ifdef YYLSP_NEEDED
  YYLTYPE yylloc;
#endif
#endif

  YYSTYPE yyval;		/*  the variable used to return		*/
				/*  semantic values from the action	*/
				/*  routines				*/

  int yylen;

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Starting parse\n");
#endif

  yystate = 0;
  yyerrstatus = 0;
  yynerrs = 0;
  yychar = YYEMPTY;		/* Cause a token to be read.  */

  /* Initialize stack pointers.
     Waste one element of value and location stack
     so that they stay on the same level as the state stack.  */

  yyssp = yyss - 1;
  yyvsp = yyvs;
#ifdef YYLSP_NEEDED
  yylsp = yyls;
#endif

/* Push a new state, which is found in  yystate  .  */
/* In all cases, when you get here, the value and location stacks
   have just been pushed. so pushing a state here evens the stacks.  */
yynewstate:

  *++yyssp = yystate;

  if (yyssp >= yyss + yystacksize - 1)
    {
      /* Give user a chance to reallocate the stack */
      /* Use copies of these so that the &'s don't force the real ones into memory. */
      YYSTYPE *yyvs1 = yyvs;
      short *yyss1 = yyss;
#ifdef YYLSP_NEEDED
      YYLTYPE *yyls1 = yyls;
#endif

      /* Get the current used size of the three stacks, in elements.  */
      int size = yyssp - yyss + 1;

#ifdef yyoverflow
      /* Each stack pointer address is followed by the size of
	 the data in use in that stack, in bytes.  */
      yyoverflow("parser stack overflow",
		 &yyss1, size * sizeof (*yyssp),
		 &yyvs1, size * sizeof (*yyvsp),
#ifdef YYLSP_NEEDED
		 &yyls1, size * sizeof (*yylsp),
#endif
		 &yystacksize);

      yyss = yyss1; yyvs = yyvs1;
#ifdef YYLSP_NEEDED
      yyls = yyls1;
#endif
#else /* no yyoverflow */
      /* Extend the stack our own way.  */
      if (yystacksize >= YYMAXDEPTH)
	{
	  yyerror("parser stack overflow");
	  return 2;
	}
      yystacksize *= 2;
      if (yystacksize > YYMAXDEPTH)
	yystacksize = YYMAXDEPTH;
      yyss = (short *) alloca (yystacksize * sizeof (*yyssp));
      __yy_bcopy ((char *)yyss1, (char *)yyss, size * sizeof (*yyssp));
      yyvs = (YYSTYPE *) alloca (yystacksize * sizeof (*yyvsp));
      __yy_bcopy ((char *)yyvs1, (char *)yyvs, size * sizeof (*yyvsp));
#ifdef YYLSP_NEEDED
      yyls = (YYLTYPE *) alloca (yystacksize * sizeof (*yylsp));
      __yy_bcopy ((char *)yyls1, (char *)yyls, size * sizeof (*yylsp));
#endif
#endif /* no yyoverflow */

      yyssp = yyss + size - 1;
      yyvsp = yyvs + size - 1;
#ifdef YYLSP_NEEDED
      yylsp = yyls + size - 1;
#endif

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Stack size increased to %d\n", yystacksize);
#endif

      if (yyssp >= yyss + yystacksize - 1)
	YYABORT;
    }

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Entering state %d\n", yystate);
#endif

 yybackup:

/* Do appropriate processing given the current state.  */
/* Read a lookahead token if we need one and don't already have one.  */
/* yyresume: */

  /* First try to decide what to do without reference to lookahead token.  */

  yyn = yypact[yystate];
  if (yyn == YYFLAG)
    goto yydefault;

  /* Not known => get a lookahead token if don't already have one.  */

  /* yychar is either YYEMPTY or YYEOF
     or a valid token in external form.  */

  if (yychar == YYEMPTY)
    {
#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Reading a token: ");
#endif
      yychar = YYLEX;
    }

  /* Convert token to internal form (in yychar1) for indexing tables with */

  if (yychar <= 0)		/* This means end of input. */
    {
      yychar1 = 0;
      yychar = YYEOF;		/* Don't call YYLEX any more */

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Now at end of input.\n");
#endif
    }
  else
    {
      yychar1 = YYTRANSLATE(yychar);

#if YYDEBUG != 0
      if (yydebug)
	{
	  fprintf (stderr, "Next token is %d (%s", yychar, yytname[yychar1]);
	  /* Give the individual parser a way to print the precise meaning
	     of a token, for further debugging info.  */
#ifdef YYPRINT
	  YYPRINT (stderr, yychar, yylval);
#endif
	  fprintf (stderr, ")\n");
	}
#endif
    }

  yyn += yychar1;
  if (yyn < 0 || yyn > YYLAST || yycheck[yyn] != yychar1)
    goto yydefault;

  yyn = yytable[yyn];

  /* yyn is what to do for this token type in this state.
     Negative => reduce, -yyn is rule number.
     Positive => shift, yyn is new state.
       New state is final state => don't bother to shift,
       just return success.
     0, or most negative number => error.  */

  if (yyn < 0)
    {
      if (yyn == YYFLAG)
	goto yyerrlab;
      yyn = -yyn;
      goto yyreduce;
    }
  else if (yyn == 0)
    goto yyerrlab;

  if (yyn == YYFINAL)
    YYACCEPT;

  /* Shift the lookahead token.  */

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Shifting token %d (%s), ", yychar, yytname[yychar1]);
#endif

  /* Discard the token being shifted unless it is eof.  */
  if (yychar != YYEOF)
    yychar = YYEMPTY;

  *++yyvsp = yylval;
#ifdef YYLSP_NEEDED
  *++yylsp = yylloc;
#endif

  /* count tokens shifted since error; after three, turn off error status.  */
  if (yyerrstatus) yyerrstatus--;

  yystate = yyn;
  goto yynewstate;

/* Do the default action for the current state.  */
yydefault:

  yyn = yydefact[yystate];
  if (yyn == 0)
    goto yyerrlab;

/* Do a reduction.  yyn is the number of a rule to reduce with.  */
yyreduce:
  yylen = yyr2[yyn];
  yyval = yyvsp[1-yylen]; /* implement default value of the action */

#if YYDEBUG != 0
  if (yydebug)
    {
      int i;

      fprintf (stderr, "Reducing via rule %d (line %d), ",
	       yyn, yyrline[yyn]);

      /* Print the symboles being reduced, and their result.  */
      for (i = yyprhs[yyn]; yyrhs[i] > 0; i++)
	fprintf (stderr, "%s ", yytname[yyrhs[i]]);
      fprintf (stderr, " -> %s\n", yytname[yyr1[yyn]]);
    }
#endif


  switch (yyn) {

case 1:
{ prog_start = yyvsp[0].stmt; ;
    break;}
case 2:
{ yyval.stmt = 0; ;
    break;}
case 3:
{
		    if (yyvsp[-1].stmt) {
			Stmt *tmp = yyvsp[-1].stmt;
			
			while (tmp->next)
			    tmp = tmp->next;
			tmp->next = yyvsp[0].stmt;
			yyval.stmt = yyvsp[-1].stmt;
		    } else
			yyval.stmt = yyvsp[0].stmt;
		;
    break;}
case 4:
{

		    yyval.stmt = alloc_stmt(STMT_COND);
		    yyval.stmt->s.cond.arms = alloc_cond_arm(yyvsp[-5].expr, yyvsp[-3].stmt);
		    yyval.stmt->s.cond.arms->next = yyvsp[-2].arm;
		    yyval.stmt->s.cond.otherwise = yyvsp[-1].stmt;
		;
    break;}
case 5:
{
		    push_loop_name(yyvsp[-4].string);
		;
    break;}
case 6:
{
		    yyval.stmt = alloc_stmt(STMT_LIST);
		    yyval.stmt->s.list.id = find_id(yyvsp[-7].string);
		    yyval.stmt->s.list.expr = yyvsp[-4].expr;
		    yyval.stmt->s.list.body = yyvsp[-1].stmt;
		    pop_loop_name();
		;
    break;}
case 7:
{
		    push_loop_name(yyvsp[-6].string);
		;
    break;}
case 8:
{
		    yyval.stmt = alloc_stmt(STMT_RANGE);
		    yyval.stmt->s.range.id = find_id(yyvsp[-9].string);
		    yyval.stmt->s.range.from = yyvsp[-6].expr;
		    yyval.stmt->s.range.to = yyvsp[-4].expr;
		    yyval.stmt->s.range.body = yyvsp[-1].stmt;
		    pop_loop_name();
		;
    break;}
case 9:
{
		    push_loop_name(0);
		;
    break;}
case 10:
{
		    yyval.stmt = alloc_stmt(STMT_WHILE);
		    yyval.stmt->s.loop.id = -1;
		    yyval.stmt->s.loop.condition = yyvsp[-4].expr;
		    yyval.stmt->s.loop.body = yyvsp[-1].stmt;
		    pop_loop_name();
		;
    break;}
case 11:
{
		    push_loop_name(yyvsp[-3].string);
		;
    break;}
case 12:
{
		    yyval.stmt = alloc_stmt(STMT_WHILE);
		    yyval.stmt->s.loop.id = find_id(yyvsp[-6].string);
		    yyval.stmt->s.loop.condition = yyvsp[-4].expr;
		    yyval.stmt->s.loop.body = yyvsp[-1].stmt;
		    pop_loop_name();
		;
    break;}
case 13:
{
		    suspend_loop_scope();
		;
    break;}
case 14:
{
		    yyval.stmt = alloc_stmt(STMT_FORK);
		    yyval.stmt->s.fork.id = -1;
		    yyval.stmt->s.fork.time = yyvsp[-4].expr;
		    yyval.stmt->s.fork.body = yyvsp[-1].stmt;
		    resume_loop_scope();
		;
    break;}
case 15:
{
		    suspend_loop_scope();
		;
    break;}
case 16:
{
		    yyval.stmt = alloc_stmt(STMT_FORK);
		    yyval.stmt->s.fork.id = find_id(yyvsp[-6].string);
		    yyval.stmt->s.fork.time = yyvsp[-4].expr;
		    yyval.stmt->s.fork.body = yyvsp[-1].stmt;
		    resume_loop_scope();
		;
    break;}
case 17:
{
		    yyval.stmt = alloc_stmt(STMT_EXPR);
		    yyval.stmt->s.expr = yyvsp[-1].expr;
		;
    break;}
case 18:
{
		    check_loop_name(0, LOOP_BREAK);
		    yyval.stmt = alloc_stmt(STMT_BREAK);
		    yyval.stmt->s.exit = -1;
		;
    break;}
case 19:
{
		    check_loop_name(yyvsp[-1].string, LOOP_BREAK);
		    yyval.stmt = alloc_stmt(STMT_BREAK);
		    yyval.stmt->s.exit = find_id(yyvsp[-1].string);
		;
    break;}
case 20:
{
		    check_loop_name(0, LOOP_CONTINUE);
		    yyval.stmt = alloc_stmt(STMT_CONTINUE);
		    yyval.stmt->s.exit = -1;
		;
    break;}
case 21:
{
		    check_loop_name(yyvsp[-1].string, LOOP_CONTINUE);
		    yyval.stmt = alloc_stmt(STMT_CONTINUE);
		    yyval.stmt->s.exit = find_id(yyvsp[-1].string);
		;
    break;}
case 22:
{
		    yyval.stmt = alloc_stmt(STMT_RETURN);
		    yyval.stmt->s.expr = yyvsp[-1].expr;
		;
    break;}
case 23:
{
		    yyval.stmt = alloc_stmt(STMT_RETURN);
		    yyval.stmt->s.expr = 0;
		;
    break;}
case 24:
{ yyval.stmt = 0; ;
    break;}
case 25:
{
		    yyval.stmt = alloc_stmt(STMT_TRY_EXCEPT);
		    yyval.stmt->s.catch.body = yyvsp[-2].stmt;
		    yyval.stmt->s.catch.excepts = yyvsp[-1].except;
		;
    break;}
case 26:
{
		    yyval.stmt = alloc_stmt(STMT_TRY_FINALLY);
		    yyval.stmt->s.finally.body = yyvsp[-3].stmt;
		    yyval.stmt->s.finally.handler = yyvsp[-1].stmt;
		;
    break;}
case 27:
{ yyval.arm = 0; ;
    break;}
case 28:
{
		    Cond_Arm *this_arm = alloc_cond_arm(yyvsp[-2].expr, yyvsp[0].stmt);
		    
		    if (yyvsp[-5].arm) {
		        Cond_Arm *tmp = yyvsp[-5].arm;

			while (tmp->next)
			    tmp = tmp->next;
			tmp->next = this_arm;
			yyval.arm = yyvsp[-5].arm;
		    } else
			yyval.arm = this_arm;
		;
    break;}
case 29:
{ yyval.stmt = 0; ;
    break;}
case 30:
{ yyval.stmt = yyvsp[0].stmt; ;
    break;}
case 31:
{ yyval.except = yyvsp[0].except; ;
    break;}
case 32:
{
		    Except_Arm *tmp = yyvsp[-1].except;
		    int		count = 1;
		    
		    while (tmp->next) {
			tmp = tmp->next;
			count++;
		    }
		    if (!tmp->codes)
			yyerror("Unreachable EXCEPT clause");
		    else if (count > 255)
			yyerror("Too many EXCEPT clauses (max. 255)");
		;
    break;}
case 33:
{
		    Except_Arm *tmp = yyvsp[-3].except;
		    
		    while (tmp->next)
			tmp = tmp->next;

		    tmp->next = yyvsp[0].except;
		    yyval.except = yyvsp[-3].except;
		;
    break;}
case 34:
{ yyval.except = alloc_except(yyvsp[-4].string ? find_id(yyvsp[-4].string) : -1, yyvsp[-2].args, yyvsp[0].stmt); ;
    break;}
case 35:
{ yyval.string = 0; ;
    break;}
case 36:
{ yyval.string = yyvsp[0].string; ;
    break;}
case 37:
{
		    yyval.expr = alloc_var(TYPE_INT);
		    yyval.expr->e.var.v.num = yyvsp[0].integer;
		;
    break;}
case 38:
{
		    yyval.expr = alloc_var(TYPE_FLOAT);
		    yyval.expr->e.var.v.fnum = yyvsp[0].real;
		;
    break;}
case 39:
{
		    yyval.expr = alloc_var(TYPE_STR);
		    yyval.expr->e.var.v.str = yyvsp[0].string;
		;
    break;}
case 40:
{
		    yyval.expr = alloc_var(TYPE_OBJ);
		    yyval.expr->e.var.v.obj = yyvsp[0].object;
		;
    break;}
case 41:
{
		    yyval.expr = alloc_var(TYPE_ERR);
		    yyval.expr->e.var.v.err = yyvsp[0].error;
		;
    break;}
case 42:
{
		    yyval.expr = alloc_expr(EXPR_ID);
		    yyval.expr->e.id = find_id(yyvsp[0].string);
		;
    break;}
case 43:
{
		    /* Treat $foo like #0.("foo") */
		    Expr *obj = alloc_var(TYPE_OBJ);
		    Expr *prop = alloc_var(TYPE_STR);
		    obj->e.var.v.obj = 0;
		    prop->e.var.v.str = yyvsp[0].string;
		    yyval.expr = alloc_binary(EXPR_PROP, obj, prop);
		;
    break;}
case 44:
{
		    /* Treat foo.bar like foo.("bar") for simplicity */
		    Expr *prop = alloc_var(TYPE_STR);
		    prop->e.var.v.str = yyvsp[0].string;
		    yyval.expr = alloc_binary(EXPR_PROP, yyvsp[-2].expr, prop);
		;
    break;}
case 45:
{
		    yyval.expr = alloc_binary(EXPR_PROP, yyvsp[-4].expr, yyvsp[-1].expr);
		;
    break;}
case 46:
{
		    /* treat foo:bar(args) like foo:("bar")(args) */
		    Expr *verb = alloc_var(TYPE_STR);
		    verb->e.var.v.str = yyvsp[-3].string;
		    yyval.expr = alloc_verb(yyvsp[-5].expr, verb, yyvsp[-1].args);
		;
    break;}
case 47:
{
		    /* treat $bar(args) like #0:("bar")(args) */
		    Expr *obj = alloc_var(TYPE_OBJ);
		    Expr *verb = alloc_var(TYPE_STR);
		    obj->e.var.v.obj = 0;
		    verb->e.var.v.str = yyvsp[-3].string;
		    yyval.expr = alloc_verb(obj, verb, yyvsp[-1].args);
		;
    break;}
case 48:
{
		    yyval.expr = alloc_verb(yyvsp[-7].expr, yyvsp[-4].expr, yyvsp[-1].args);
		;
    break;}
case 49:
{
		    dollars_ok--;
		    yyval.expr = alloc_binary(EXPR_INDEX, yyvsp[-4].expr, yyvsp[-1].expr);
		;
    break;}
case 50:
{
		    dollars_ok--;
		    yyval.expr = alloc_expr(EXPR_RANGE);
		    yyval.expr->e.range.base = yyvsp[-6].expr;
		    yyval.expr->e.range.from = yyvsp[-3].expr;
		    yyval.expr->e.range.to = yyvsp[-1].expr;
		;
    break;}
case 51:
{
		    if (!dollars_ok)
  			yyerror("Illegal context for `$' expression.");

			  yyval.expr = alloc_expr(EXPR_LENGTH);

		;
    break;}
case 52:
{
		    Expr *e = yyvsp[-2].expr;

		    if (e->kind == EXPR_LIST) {
			e->kind = EXPR_SCATTER;
			if (e->e.list) {
			    e->e.scatter = scatter_from_arglist(e->e.list);
			    vet_scatter(e->e.scatter);
			} else
			    yyerror("Empty list in scattering assignment.");
		    } else {
			if (e->kind == EXPR_RANGE)
			    e = e->e.range.base;
			while (e->kind == EXPR_INDEX)
			    e = e->e.bin.lhs;
			if (e->kind != EXPR_ID  &&  e->kind != EXPR_PROP)
			    yyerror("Illegal expression on left side of"
				    " assignment.");
		    }
		    yyval.expr = alloc_binary(EXPR_ASGN, yyvsp[-2].expr, yyvsp[0].expr);
	        ;
    break;}
case 53:
{
		    Expr       *e = alloc_expr(EXPR_SCATTER);

		    e->e.scatter = yyvsp[-3].scatter;
		    vet_scatter(yyvsp[-3].scatter);
		    yyval.expr = alloc_binary(EXPR_ASGN, e, yyvsp[0].expr);
		;
    break;}
case 54:
{
		    unsigned f_no;

		    yyval.expr = alloc_expr(EXPR_CALL);
		    if ((f_no = number_func_by_name(yyvsp[-3].string)) == FUNC_NOT_FOUND) {
			/* Replace with call_function("$1", @args) */
			Expr	       *fname = alloc_var(TYPE_STR);
			Arg_List       *a = alloc_arg_list(ARG_NORMAL, fname);

			fname->e.var.v.str = yyvsp[-3].string;
			a->next = yyvsp[-1].args;
			warning("Unknown built-in function: ", yyvsp[-3].string);
			yyval.expr->e.call.func = number_func_by_name("call_function");
			yyval.expr->e.call.args = a;
		    } else {
			yyval.expr->e.call.func = f_no;
			yyval.expr->e.call.args = yyvsp[-1].args;
			dealloc_string(yyvsp[-3].string);
		    }
		;
    break;}
case 55:
{
		    yyval.expr = alloc_binary(EXPR_PLUS, yyvsp[-2].expr, yyvsp[0].expr);
		;
    break;}
case 56:
{
		    yyval.expr = alloc_binary(EXPR_MINUS, yyvsp[-2].expr, yyvsp[0].expr);
		;
    break;}
case 57:
{
		    yyval.expr = alloc_binary(EXPR_TIMES, yyvsp[-2].expr, yyvsp[0].expr);
		;
    break;}
case 58:
{
		    yyval.expr = alloc_binary(EXPR_DIVIDE, yyvsp[-2].expr, yyvsp[0].expr);
		;
    break;}
case 59:
{
		    yyval.expr = alloc_binary(EXPR_MOD, yyvsp[-2].expr, yyvsp[0].expr);
		;
    break;}
case 60:
{
		    yyval.expr = alloc_binary(EXPR_EXP, yyvsp[-2].expr, yyvsp[0].expr);
		;
    break;}
case 61:
{
		    yyval.expr = alloc_binary(EXPR_AND, yyvsp[-2].expr, yyvsp[0].expr);
		;
    break;}
case 62:
{
		    yyval.expr = alloc_binary(EXPR_OR, yyvsp[-2].expr, yyvsp[0].expr);
		;
    break;}
case 63:
{
		    yyval.expr = alloc_binary(EXPR_EQ, yyvsp[-2].expr, yyvsp[0].expr);
		;
    break;}
case 64:
{
		    yyval.expr = alloc_binary(EXPR_NE, yyvsp[-2].expr, yyvsp[0].expr);
		;
    break;}
case 65:
{
		    yyval.expr = alloc_binary(EXPR_LT, yyvsp[-2].expr, yyvsp[0].expr);
		;
    break;}
case 66:
{
		    yyval.expr = alloc_binary(EXPR_LE, yyvsp[-2].expr, yyvsp[0].expr);
		;
    break;}
case 67:
{
		    yyval.expr = alloc_binary(EXPR_GT, yyvsp[-2].expr, yyvsp[0].expr);
		;
    break;}
case 68:
{
		    yyval.expr = alloc_binary(EXPR_GE, yyvsp[-2].expr, yyvsp[0].expr);
		;
    break;}
case 69:
{
		    yyval.expr = alloc_binary(EXPR_IN, yyvsp[-2].expr, yyvsp[0].expr);
		;
    break;}
case 70:
{
		    if (yyvsp[0].expr->kind == EXPR_VAR
			&& (yyvsp[0].expr->e.var.type == TYPE_INT
			    || yyvsp[0].expr->e.var.type == TYPE_FLOAT)) {
			switch (yyvsp[0].expr->e.var.type) {
			  case TYPE_INT:
			    yyvsp[0].expr->e.var.v.num = -yyvsp[0].expr->e.var.v.num;
			    break;
			  case TYPE_FLOAT:
			    *(yyvsp[0].expr->e.var.v.fnum) = - (*(yyvsp[0].expr->e.var.v.fnum));
			    break;
			  default:
			    break;
			}
		        yyval.expr = yyvsp[0].expr;
		    } else {
			yyval.expr = alloc_expr(EXPR_NEGATE);
			yyval.expr->e.expr = yyvsp[0].expr;
		    }
		;
    break;}
case 71:
{
		    yyval.expr = alloc_expr(EXPR_NOT);
		    yyval.expr->e.expr = yyvsp[0].expr;
		;
    break;}
case 72:
{ yyval.expr = yyvsp[-1].expr; ;
    break;}
case 73:
{
		    yyval.expr = alloc_expr(EXPR_LIST);
		    yyval.expr->e.list = yyvsp[-1].args;
		;
    break;}
case 74:
{
		    yyval.expr = alloc_expr(EXPR_COND);
		    yyval.expr->e.cond.condition = yyvsp[-4].expr;
		    yyval.expr->e.cond.consequent = yyvsp[-2].expr;
		    yyval.expr->e.cond.alternate = yyvsp[0].expr;
		;
    break;}
case 75:
{
		    yyval.expr = alloc_expr(EXPR_CATCH);
		    yyval.expr->e.catch.try = yyvsp[-4].expr;
		    yyval.expr->e.catch.codes = yyvsp[-2].args;
		    yyval.expr->e.catch.except = yyvsp[-1].expr;
		;
    break;}
case 76:
{ dollars_ok++; ;
    break;}
case 77:
{ yyval.args = 0; ;
    break;}
case 78:
{ yyval.args = yyvsp[0].args; ;
    break;}
case 79:
{ yyval.expr = 0; ;
    break;}
case 80:
{ yyval.expr = yyvsp[0].expr; ;
    break;}
case 81:
{ yyval.args = 0; ;
    break;}
case 82:
{ yyval.args = yyvsp[0].args; ;
    break;}
case 83:
{ yyval.args = alloc_arg_list(ARG_NORMAL, yyvsp[0].expr); ;
    break;}
case 84:
{ yyval.args = alloc_arg_list(ARG_SPLICE, yyvsp[0].expr); ;
    break;}
case 85:
{
		    Arg_List *this_arg = alloc_arg_list(ARG_NORMAL, yyvsp[0].expr);

		    if (yyvsp[-2].args) {
			Arg_List *tmp = yyvsp[-2].args;

			while (tmp->next)
			    tmp = tmp->next;
			tmp->next = this_arg;
			yyval.args = yyvsp[-2].args;
		    } else
			yyval.args = this_arg;
		;
    break;}
case 86:
{
		    Arg_List *this_arg = alloc_arg_list(ARG_SPLICE, yyvsp[0].expr);

		    if (yyvsp[-3].args) {
			Arg_List *tmp = yyvsp[-3].args;

			while (tmp->next)
			    tmp = tmp->next;
			tmp->next = this_arg;
			yyval.args = yyvsp[-3].args;
		    } else
			yyval.args = this_arg;
		;
    break;}
case 87:
{
		    Scatter    *sc = scatter_from_arglist(yyvsp[-2].args);

		    if (sc)
			yyval.scatter = add_scatter_item(sc, yyvsp[0].scatter);
		    else
			yyval.scatter = yyvsp[0].scatter;
		;
    break;}
case 88:
{
		    yyval.scatter = add_scatter_item(yyvsp[-2].scatter, yyvsp[0].scatter);
		;
    break;}
case 89:
{
		    yyval.scatter = add_scatter_item(yyvsp[-2].scatter, alloc_scatter(SCAT_REQUIRED,
							    find_id(yyvsp[0].string), 0));
		;
    break;}
case 90:
{
		    yyval.scatter = add_scatter_item(yyvsp[-3].scatter, alloc_scatter(SCAT_REST,
							    find_id(yyvsp[0].string), 0));
		;
    break;}
case 91:
{   yyval.scatter = yyvsp[0].scatter;  ;
    break;}
case 92:
{
		    yyval.scatter = alloc_scatter(SCAT_OPTIONAL, find_id(yyvsp[0].string), 0);
		;
    break;}
case 93:
{
		    yyval.scatter = alloc_scatter(SCAT_OPTIONAL, find_id(yyvsp[-2].string), yyvsp[0].expr);
		;
    break;}
}
   /* the action file gets copied in in place of this dollarsign */

  yyvsp -= yylen;
  yyssp -= yylen;
#ifdef YYLSP_NEEDED
  yylsp -= yylen;
#endif

#if YYDEBUG != 0
  if (yydebug)
    {
      short *ssp1 = yyss - 1;
      fprintf (stderr, "state stack now");
      while (ssp1 != yyssp)
	fprintf (stderr, " %d", *++ssp1);
      fprintf (stderr, "\n");
    }
#endif

  *++yyvsp = yyval;

#ifdef YYLSP_NEEDED
  yylsp++;
  if (yylen == 0)
    {
      yylsp->first_line = yylloc.first_line;
      yylsp->first_column = yylloc.first_column;
      yylsp->last_line = (yylsp-1)->last_line;
      yylsp->last_column = (yylsp-1)->last_column;
      yylsp->text = 0;
    }
  else
    {
      yylsp->last_line = (yylsp+yylen-1)->last_line;
      yylsp->last_column = (yylsp+yylen-1)->last_column;
    }
#endif

  /* Now "shift" the result of the reduction.
     Determine what state that goes to,
     based on the state we popped back to
     and the rule number reduced by.  */

  yyn = yyr1[yyn];

  yystate = yypgoto[yyn - YYNTBASE] + *yyssp;
  if (yystate >= 0 && yystate <= YYLAST && yycheck[yystate] == *yyssp)
    yystate = yytable[yystate];
  else
    yystate = yydefgoto[yyn - YYNTBASE];

  goto yynewstate;

yyerrlab:   /* here on detecting error */

  if (! yyerrstatus)
    /* If not already recovering from an error, report this error.  */
    {
      ++yynerrs;

#ifdef YYERROR_VERBOSE
      yyn = yypact[yystate];

      if (yyn > YYFLAG && yyn < YYLAST)
	{
	  int size = 0;
	  char *msg;
	  int x, count;

	  count = 0;
	  for (x = 0; x < (sizeof(yytname) / sizeof(char *)); x++)
	    if (yycheck[x + yyn] == x)
	      size += strlen(yytname[x]) + 15, count++;
	  msg = (char *) malloc(size + 15);
	  if (msg != 0)
	    {
	      strcpy(msg, "parse error");

	      if (count < 5)
		{
		  count = 0;
		  for (x = 0; x < (sizeof(yytname) / sizeof(char *)); x++)
		    if (yycheck[x + yyn] == x)
		      {
			strcat(msg, count == 0 ? ", expecting `" : " or `");
			strcat(msg, yytname[x]);
			strcat(msg, "'");
			count++;
		      }
		}
	      yyerror(msg);
	      free(msg);
	    }
	  else
	    yyerror ("parse error; also virtual memory exceeded");
	}
      else
#endif /* YYERROR_VERBOSE */
	yyerror("parse error");
    }

yyerrlab1:   /* here on error raised explicitly by an action */

  if (yyerrstatus == 3)
    {
      /* if just tried and failed to reuse lookahead token after an error, discard it.  */

      /* return failure if at end of input */
      if (yychar == YYEOF)
	YYABORT;

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Discarding token %d (%s).\n", yychar, yytname[yychar1]);
#endif

      yychar = YYEMPTY;
    }

  /* Else will try to reuse lookahead token
     after shifting the error token.  */

  yyerrstatus = 3;		/* Each real token shifted decrements this */

  goto yyerrhandle;

yyerrdefault:  /* current state does not do anything special for the error token. */

#if 0
  /* This is wrong; only states that explicitly want error tokens
     should shift them.  */
  yyn = yydefact[yystate];  /* If its default is to accept any token, ok.  Otherwise pop it.*/
  if (yyn) goto yydefault;
#endif

yyerrpop:   /* pop the current state because it cannot handle the error token */

  if (yyssp == yyss) YYABORT;
  yyvsp--;
  yystate = *--yyssp;
#ifdef YYLSP_NEEDED
  yylsp--;
#endif

#if YYDEBUG != 0
  if (yydebug)
    {
      short *ssp1 = yyss - 1;
      fprintf (stderr, "Error: state stack now");
      while (ssp1 != yyssp)
	fprintf (stderr, " %d", *++ssp1);
      fprintf (stderr, "\n");
    }
#endif

yyerrhandle:

  yyn = yypact[yystate];
  if (yyn == YYFLAG)
    goto yyerrdefault;

  yyn += YYTERROR;
  if (yyn < 0 || yyn > YYLAST || yycheck[yyn] != YYTERROR)
    goto yyerrdefault;

  yyn = yytable[yyn];
  if (yyn < 0)
    {
      if (yyn == YYFLAG)
	goto yyerrpop;
      yyn = -yyn;
      goto yyreduce;
    }
  else if (yyn == 0)
    goto yyerrpop;

  if (yyn == YYFINAL)
    YYACCEPT;

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Shifting error token, ");
#endif

  *++yyvsp = yylval;
#ifdef YYLSP_NEEDED
  *++yylsp = yylloc;
#endif

  yystate = yyn;
  goto yynewstate;
}


static int		lineno, nerrors, must_rename_keywords;
static Parser_Client	client;
static void	       *client_data;
static Names	       *local_names;

static int
find_id(char *name)
{
    int slot = find_or_add_name(&local_names, name);

    dealloc_string(name);
    return slot;
}

static void
yyerror(const char *s)
{
	error(s, 0);
}

static const char *
fmt_error(const char *s, const char *t)
{
    static Stream      *str = 0;

    if (str == 0)
	str = new_stream(100);
    if (t)
	stream_printf(str, "Line %d:  %s%s", lineno, s, t);
    else
	stream_printf(str, "Line %d:  %s", lineno, s);
    return reset_stream(str);
}

static void
error(const char *s, const char *t)
{
    nerrors++;
    (*(client.error))(client_data, fmt_error(s, t));
}

static void
warning(const char *s, const char *t)
{
    if (client.warning)
	(*(client.warning))(client_data, fmt_error(s, t));
    else
	error(s, t);
}

static int unget_buffer[5], unget_count;

static int
lex_getc(void)
{
    if (unget_count > 0)
	return unget_buffer[--unget_count];
    else
	return (*(client.getch))(client_data);
}

static void
lex_ungetc(int c)
{
    unget_buffer[unget_count++] = c;
}

static int
follow(int expect, int ifyes, int ifno)     /* look ahead for >=, etc. */
{
    int c = lex_getc();

    if (c == expect)
        return ifyes;
    lex_ungetc(c);
    return ifno;
}

static Stream  *token_stream = 0;

static int
yylex(void)
{
    int		c;
//	char c;

    reset_stream(token_stream);

start_over:

    do {
	c = lex_getc();
	if (c == '\n') lineno++;
    } while (isspace(c));

    if (c == '/') {
	c = lex_getc();
	if (c == '*') {
	    for (;;) {
		c = lex_getc();
		if (c == '*') {
		    c = lex_getc();
		    if (c == '/')
			goto start_over;
		}
		if (c == EOF) {
		    yyerror("End of program while in a comment");
		    return c;
		}
	    }
	} else {
	    lex_ungetc(c);
	    return '/';
	}
    }

    if (c == '#') {
	int            negative = 0;
	Objid          oid = 0;

	c = lex_getc();
	if (c == '-') {
	    negative = 1;
	    c = lex_getc();
	}
	if (!isdigit(c)) {
	    yyerror("Malformed object number");
	    lex_ungetc(c);
	    return 0;
	}
	do {
	    oid = oid * 10 + (c - '0');
	    c = lex_getc();
	} while (isdigit(c));
	lex_ungetc(c);

	yylval.object = negative ? -oid : oid;
	return tOBJECT;
    }

    if (isdigit(c) || (c == '.'  &&  language_version >= DBV_Float)) {
	int	n = 0;
	int	type = tINTEGER;

	while (isdigit(c)) {
	    n = n * 10 + (c - '0');
	    stream_add_char(token_stream, (char) c);
	    c = lex_getc();
	}

	if (language_version >= DBV_Float && c == '.') {
	    /* maybe floating-point (but maybe `..') */
	    int cc;

	    lex_ungetc(cc = lex_getc()); /* peek ahead */
	    if (isdigit(cc)) {	/* definitely floating-point */
		type = tFLOAT;
		do {
		    stream_add_char(token_stream, (char) c);
		    c = lex_getc();
		} while (isdigit(c));
	    } else if (stream_length(token_stream) == 0)
		/* no digits before or after `.'; not a number at all */
		goto normal_dot;
	    else if (cc != '.') {
		/* Some digits before dot, not `..' */
		type = tFLOAT;
		stream_add_char(token_stream, (char) c);
		c = lex_getc();
	    }
	}

	if (language_version >= DBV_Float && (c == 'e' || c == 'E')) {
	    /* better be an exponent */
	    type = tFLOAT;
	    stream_add_char(token_stream, (char) c);
	    c = lex_getc();
	    if (c == '+' || c == '-') {
		stream_add_char(token_stream, (char) c);
		c = lex_getc();
	    }
	    if (!isdigit(c)) {
		yyerror("Malformed floating-point literal");
		lex_ungetc(c);
		return 0;
	    }
	    do {
		stream_add_char(token_stream, (char) c);
		c = lex_getc();
	    } while (isdigit(c));
	}
	
	lex_ungetc(c);

	if (type == tINTEGER)
	    yylval.integer = n;
	else {
	    double	d;
	    
	    d = strtod(reset_stream(token_stream), 0);
	    if (!IS_REAL(d)) {
		yyerror("Floating-point literal out of range");
		d = 0.0;
	    }
	    yylval.real = alloc_float(d);
	}
	return type;
    }
    
    if (isalpha(c) || c == '_') {
	char	       *buf;
	Keyword	       *k;

	stream_add_char(token_stream, (char) c);
	while (isalnum(c = lex_getc()) || c == '_')
	    stream_add_char(token_stream, (char) c);
	lex_ungetc(c);
	buf = reset_stream(token_stream);

	k = find_keyword(buf);
	if (k) {
	    if (k->version <= language_version) {
		int	t = k->token;

		if (t == tERROR)
		    yylval.error = k->error;
		return t;
	    } else {		/* New keyword being used as an identifier */
		if (!must_rename_keywords)
		    warning("Renaming old use of new keyword: ", buf);
		must_rename_keywords = 1;
	    }
	}
	
	yylval.string = alloc_string(buf);
	return tID;
    }

    if (c == '"') {
	while(1) {
	    c = lex_getc();
	    if (c == '"')
		break;
	    if (c == '\\')
		c = lex_getc();
	    if (c == '\n' || c == EOF) {
		yyerror("Missing quote");
		break;
	    }
	    stream_add_char(token_stream, (char) c);
	}
	yylval.string = alloc_string(reset_stream(token_stream));
	return tSTRING;
    }

    switch(c) {
      case '>':         return follow('=', tGE, '>');
      case '<':         return follow('=', tLE, '<');
      case '=':         return ((c = follow('=', tEQ, 0))
				? c
				: follow('>', tARROW, '='));
      case '!':         return follow('=', tNE, '!');
      case '|':         return follow('|', tOR, '|');
      case '&':         return follow('&', tAND, '&');
      normal_dot:
      case '.':		return follow('.', tTO, '.');
      default:          return c;
    }
}

static Scatter *
add_scatter_item(Scatter *first, Scatter *last)
{
    Scatter    *tmp = first;

    while (tmp->next)
	tmp = tmp->next;
    tmp->next = last;

    return first;
}

static Scatter *
scatter_from_arglist(Arg_List *a)
{
    Scatter    *sc = 0, **scp;
    Arg_List   *anext;

    for (scp = &sc; a; a = anext, scp = &((*scp)->next)) {
	if (a->expr->kind == EXPR_ID) {
	    *scp = alloc_scatter(a->kind == ARG_NORMAL ? SCAT_REQUIRED
						       : SCAT_REST,
				 a->expr->e.id, 0);
	    anext = a->next;
	    dealloc_node(a->expr);
	    dealloc_node(a);
	} else {
	    yyerror("Scattering assignment targets must be simple variables.");
	    return 0;
	}
    }

    return sc;
}

static void
vet_scatter(Scatter *sc)
{
    int	seen_rest = 0, count = 0;

    for (; sc; sc = sc->next) {
	if (sc->kind == SCAT_REST) {
	    if (seen_rest)
		yyerror("More than one `@' target in scattering assignment.");
	    else
		seen_rest = 1;
	}
	count++;
    }

    if (count > 255)
	yyerror("Too many targets in scattering assignment.");
}

struct loop_entry {
    struct loop_entry  *next;
    const char	       *name;
    int			is_barrier;
};

static struct loop_entry *loop_stack;

static void
push_loop_name(const char *name)
{
    struct loop_entry  *entry = mymalloc(sizeof(struct loop_entry), M_AST);

    entry->next = loop_stack;
    entry->name = (name ? str_dup(name) : 0);
    entry->is_barrier = 0;
    loop_stack = entry;
}

static void
pop_loop_name(void)
{
    if (!loop_stack)
	errlog("PARSER: Empty loop stack in POP_LOOP_NAME!\n");
    else if (loop_stack->is_barrier)
	errlog("PARSER: Tried to pop loop-scope barrier!\n");
    else {
	struct loop_entry      *entry = loop_stack;

	loop_stack = loop_stack->next;
	if (entry->name)
	    free_str(entry->name);
	myfree(entry, M_AST);
    }
}

static void
suspend_loop_scope(void)
{
    struct loop_entry  *entry = mymalloc(sizeof(struct loop_entry), M_AST);

    entry->next = loop_stack;
    entry->name = 0;
    entry->is_barrier = 1;
    loop_stack = entry;
}

static void
resume_loop_scope(void)
{
    if (!loop_stack)
	errlog("PARSER: Empty loop stack in RESUME_LOOP_SCOPE!\n");
    else if (!loop_stack->is_barrier)
	errlog("PARSER: Tried to resume non-loop-scope barrier!\n");
    else {
	struct loop_entry      *entry = loop_stack;

	loop_stack = loop_stack->next;
	myfree(entry, M_AST);
    }
}

static void
check_loop_name(const char *name, enum loop_exit_kind kind)
{
    struct loop_entry  *entry;

    if (!name) {
	if (!loop_stack  ||  loop_stack->is_barrier) {
	    if (kind == LOOP_BREAK)
		yyerror("No enclosing loop for `break' statement");
	    else
		yyerror("No enclosing loop for `continue' statement");
	}
	return;
    }
    
    for (entry = loop_stack; entry && !entry->is_barrier; entry = entry->next)
	if (entry->name  &&  mystrcasecmp(entry->name, name) == 0)
	    return;

    if (kind == LOOP_BREAK)
	error("Invalid loop name in `break' statement: ", name);
    else
	error("Invalid loop name in `continue' statement: ", name);
}

Program *
parse_program(DB_Version version, Parser_Client c, void *data)
{
    extern int	yyparse();
    Program    *prog;
    
    if (token_stream == 0)
	token_stream = new_stream(1024);
    unget_count = 0;
    nerrors = 0;
    must_rename_keywords = 0;
    lineno = 1;
    client = c;
    client_data = data;
    local_names = new_builtin_names(version);
    dollars_ok = 0;
    loop_stack = 0;
    language_version = version;
    
    begin_code_allocation();
    yyparse();
    end_code_allocation(nerrors > 0);
    if (loop_stack) {
	if (nerrors == 0)
	    errlog("PARSER: Non-empty loop-scope stack!\n");
	while (loop_stack) {
	    struct loop_entry *entry = loop_stack;

	    loop_stack = loop_stack->next;
	    if (entry->name)
		free_str(entry->name);
	    myfree(entry, M_AST);
	}
    }
    
    if (nerrors == 0) {
	if (must_rename_keywords) {
	    /* One or more new keywords were used as identifiers in this code,
	     * possibly as local variable names (but maybe only as property or
	     * verb names).  Such local variables must be renamed to avoid a
	     * conflict in the new world.  We just add underscores to the end
	     * until it stops conflicting with any other local variable.
	     */
	    unsigned i;
		char *name;

	    for (i = first_user_slot(version); i < local_names->size; i++) {
		name = (char *) local_names->names[i];
	    
		if (find_keyword(name)) { /* Got one... */
		    stream_add_string(token_stream, name);
		    do {
			stream_add_char(token_stream, '_');
		    } while (find_name(local_names,
				       stream_contents(token_stream)) >= 0);
		    free_str(name);
		    local_names->names[i] =
			str_dup(reset_stream(token_stream));
		}
	    }
	}

	prog = generate_code(prog_start, version);
	prog->num_var_names = local_names->size;
	prog->var_names = local_names->names;

	myfree(local_names, M_NAMES);
	free_stmt(prog_start);

	return prog;
    } else {
	free_names(local_names);
	return 0;
    }
}

struct parser_state {
    Var		code;		/* a list of strings */
    int		cur_string;	/* which string? */
    int		cur_char;	/* which character in that string? */
    Var		errors;		/* a list of strings */
};

static void
my_error(void *data, char *msg)
{
    struct parser_state	*state = (struct parser_state *) data;
    Var			v;
    
    v.type = TYPE_STR;
    v.v.str = str_dup(msg);
    state->errors = listappend(state->errors, v);
}

static int
my_getc(void *data)
{
    struct parser_state	*state = (struct parser_state *) data;
    Var			code;
    char		c;
    
    code = state->code;
    if (current_task_has_timed_out()  ||
        state->cur_string > code.v.list[0].v.num)
	return EOF;
    else if (!(c = code.v.list[state->cur_string].v.str[state->cur_char])) {
	state->cur_string++;
	state->cur_char = 0;
	return '\n';
    } else {
	state->cur_char++;
	return c;
    }
}

static Parser_Client list_parser_client = { my_error, 0, my_getc };

Program *
parse_list_as_program(Var code, Var *errors)
{
    struct parser_state	state;
    Program	       *program;
    
    state.code = code;
    state.cur_string = 1;
    state.cur_char = 0;
    state.errors = new_list(0);
    program = parse_program(current_version, list_parser_client, &state);
    *errors = state.errors;

    return program;
}

char rcsid_parser[] = "$Id: parser.y,v 2.11 1996/05/12 21:28:37 pavel Exp $";

/* $Log: parser.y,v $
 * Revision 2.11  1996/05/12  21:28:37  pavel
 * Removed non-backward-compatible parse error for negating a non-numeric
 * literal.  Made certain aspects of floating-point literal parsing be
 * language-version-dependent, to maintain backward compatibility with DBs
 * written by pre-float servers.  Release 1.8.0p5.
 *
 * Revision 2.10  1996/04/19  01:26:38  pavel
 * Fixed potential memory smash when `$' expression occurs in an illegal
 * context.  Release 1.8.0p4.
 *
 * Revision 2.9  1996/03/10  01:08:01  pavel
 * Removed bogus `call_function("foo", ...)' --> `foo(...)' conversion.
 * Fixed line numbers of renamed-variable warnings.  Release 1.8.0.
 *
 * Revision 2.8  1996/02/18  23:14:54  pavel
 * Liberalized parsing of floating-point numbers to accept `3.' and `3.e1'.
 * Changed parsing to replace call_function("foo", @args) with foo(@args) if
 * foo is a known function.  Release 1.8.0beta3.
 *
 * Revision 2.7  1996/02/11  00:46:01  pavel
 * Liberalized parsing of floating-point literals to accept `.02' and the
 * like.  Fixed a bug where `1E6' and the like would be parsed as an integer.
 * Release 1.8.0beta2.
 *
 * Revision 2.6  1996/02/08  06:32:48  pavel
 * Added support for floating-point literals, exponentiation expression, named
 * WHILE loops, and BREAK and CONTINUE statements.  Generalized support for
 * version-dependent compilation.  Renamed TYPE_NUM to TYPE_INT.  Removed use
 * of ungetch() method on Parser_Client.  Updated copyright notice for 1996.
 * Release 1.8.0beta1.
 *
 * Revision 2.5  1996/01/16  07:17:05  pavel
 * Add support for scattering assignment.  Release 1.8.0alpha6.
 *
 * Revision 2.4  1996/01/11  07:50:49  pavel
 * Added missing #include "my-string.h".  Release 1.8.0alpha5.
 *
 * Revision 2.3  1995/12/31  00:09:40  pavel
 * Removed unused tokens tRAISE and tFORLIST (huh?).  Added `$' expression.
 * Release 1.8.0alpha4.
 *
 * Revision 2.2  1995/12/28  00:43:25  pavel
 * Added some support for MOO-compilation warnings.  Changed `unknown built-in
 * function' error into a warning with recovery.  Changed identifier-renaming
 * log message into a warning.  Release 1.8.0alpha3.
 *
 * Revision 2.1  1995/12/11  08:01:26  pavel
 * Added support for `$foo(...)' syntax.  Release 1.8.0alpha2.
 *
 * Revision 2.0  1995/11/30  04:48:11  pavel
 * New baseline version, corresponding to release 1.8.0alpha1.
 *
 * Revision 1.13  1992/10/23  23:03:47  pavel
 * Added copyright notice.
 *
 * Revision 1.12  1992/10/23  22:21:43  pavel
 * Eliminated all uses of the useless macro NULL.
 *
 * Revision 1.11  1992/10/21  03:02:35  pavel
 * Converted to use new automatic configuration system.
 *
 * Revision 1.10  1992/08/31  22:25:51  pjames
 * Changed some `char *'s to `const char *'
 * 
 * Revision 1.9  1992/08/28  23:18:38  pjames
 * Added ASGN_RANGE parsing code.  Fixed tiny memory leak.
 * 
 * Revision 1.8  1992/08/28  16:16:52  pjames
 * Changed `ak_dealloc_string()' to `free_str()'.
 * 
 * Revision 1.7  1992/08/12  01:49:58  pjames
 * Negative literals are now accepted (instead of negated positive literals).
 * 
 * Revision 1.6  1992/08/10  16:55:59  pjames
 * Updated #includes.
 * 
 * Revision 1.5  1992/07/30  21:22:08  pjames
 * Removed max_stack calculations (moved to vector.c).
 * 
 * Revision 1.4  1992/07/29  18:05:12  pjames
 * $$ no longer = NULL when there is an illegal left hand side of an
 * expression.
 * 
 * Revision 1.3  1992/07/27  18:12:41  pjames
 * Changed name of ct_env to var_names and const_env to literals.
 * Modified call to vectorize to use new argument format.
 * Assignment statements now check left hand side properly for allowable
 * expressions, and now frees memory from reused expressions.
 * 
 * Revision 1.2  1992/07/21  00:05:16  pavel
 * Added rcsid_<filename-root> declaration to hold the RCS ident. string.
 * 
 * Revision 1.1  1992/07/20  23:23:12  pavel
 * Initial RCS-controlled version.
 */
